
#include "EvaluationScope.hpp"
#include <memory>

EvaluationScope::EvaluationScope(unordered_map<int, shared_ptr<EvalValue>> *scriptVariables, int deepestScope) {
    _scriptScope = scriptVariables;
    _localScope = vector<unordered_map<int, shared_ptr<EvalValue>>>(deepestScope);
    _currentScope = -1;
}

void EvaluationScope::CreateVariable(int scope, int id, shared_ptr<EvalValue> value) {
    if (scope == 0){
        _scriptScope->insert_or_assign(id, value);
    } else{
        _localScope[scope - 1].insert_or_assign(id, value);
    }
}

void EvaluationScope::SetVariable(int scope, int id, shared_ptr<EvalValue> value) {
    if (scope == 0){
        _scriptScope->insert_or_assign(id, value);
    } else{
        _localScope[scope - 1][id] = value;
    }
}

shared_ptr<EvalValue> EvaluationScope::GetVariable(int scope, int id) {
    if (scope == 0){
        return _scriptScope->at(id);
    }
    return _localScope[scope - 1][id];
}

void EvaluationScope::OuterScope() {
    _currentScope++;
}

void EvaluationScope::InnerScope() {
    auto scope = this->_localScope[_currentScope];
    scope.clear();
    _currentScope--;

}
