
#ifndef PORYGONLANG_EVALUATOR_HPP
#define PORYGONLANG_EVALUATOR_HPP

#include <string>
#include "../Binder/BoundStatements/BoundStatement.hpp"
#include "../Script.hpp"
#include "EvalValues/EvalValue.hpp"
#include "EvalValues/NumericEvalValue.hpp"
#include "EvalValues/StringEvalValue.hpp"
#include "EvalValues/StringEvalValue.hpp"
#include "EvaluationScope/EvaluationScope.hpp"

using namespace std;

class Evaluator {
    shared_ptr<EvalValue> _result;
    shared_ptr<EvalValue> _lastValue;

    Script* _scriptData;
    EvaluationScope* _evaluationScope;

    void EvaluateStatement(BoundStatement* statement);
    void EvaluateExpressionStatement(BoundExpressionStatement* statement);
    void EvaluateAssignmentStatement(BoundAssignmentStatement* statement);
    void EvaluateFunctionDeclarationStatement(BoundFunctionDeclarationStatement *statement);

    shared_ptr<EvalValue> EvaluateExpression(BoundExpression* expression);
    shared_ptr<NumericEvalValue> EvaluateIntegerExpression(BoundExpression* expression);
    shared_ptr<BooleanEvalValue> EvaluateBoolExpression(BoundExpression* expression);
    shared_ptr<StringEvalValue> EvaluateStringExpression(BoundExpression* expression);
    shared_ptr<EvalValue> EvaluateFunctionExpression(BoundExpression *expression);
    shared_ptr<EvalValue>EvaluateNilExpression(BoundExpression *expression);

    shared_ptr<NumericEvalValue> EvaluateIntegerBinary(BoundBinaryExpression* expression);
    shared_ptr<BooleanEvalValue> EvaluateBooleanBinary(BoundBinaryExpression *expression);
    shared_ptr<StringEvalValue> EvaluateStringBinary(BoundBinaryExpression *expression);

    shared_ptr<NumericEvalValue> EvaluateIntegerUnary(BoundUnaryExpression* expression);
    shared_ptr<BooleanEvalValue> EvaluateBooleanUnary(BoundUnaryExpression *expression);
    shared_ptr<EvalValue> EvaluateFunctionCallExpression(BoundExpression *expression);

    shared_ptr<EvalValue> GetVariable(BoundVariableExpression *expression);
public:
    explicit Evaluator(Script* script){
        _scriptData = script;
        _evaluationScope = nullptr;
    }

    ~Evaluator(){
        delete _evaluationScope;
    }

    void Evaluate(BoundScriptStatement* statement);
    void EvaluateBlockStatement(BoundBlockStatement* statement);

    EvaluationScope* GetScope(){
        return _evaluationScope;
    }

    EvalValue* GetLastValue(){
        return _lastValue.get();
    }
};



#endif //PORYGONLANG_EVALUATOR_HPP
